@@ -73,31 +73,118 @@ class UpnpRequester(ABC):
 class UpnpDevice:
     """UPnP Device representation."""
 
+    # pylint: disable=too-many-public-methods
+
     def __init__(
         self,
         requester: UpnpRequester,
         device_info: DeviceInfo,
         services: Sequence["UpnpService"],
+        embedded_devices: Sequence["UpnpDevice"],
     ) -> None:
         """Initialize."""
         # pylint: disable=too-many-arguments
         self.requester = requester
         self.device_info = device_info
         self.services = {service.service_type: service for service in services}
+        self.embedded_devices = {
+            embedded_device.device_type: embedded_device
+            for embedded_device in embedded_devices
+        }
+        self._parent_device: Optional[UpnpDevice] = None
 
         # bind services to ourselves
         for service in services:
             service.device = self
 
+        # bind devices to ourselves
+        for embedded_device in embedded_devices:
+            embedded_device.parent_device = self
+
         # SSDP headers.
         self.ssdp_headers: SsdpHeaders = CaseInsensitiveDict()
 
         # Just initialized, mark available.
         self.available = True
 
-    def reinit(self, device: "UpnpDevice") -> None:
+    @property
+    def parent_device(self) -> Optional["UpnpDevice"]:
+        """Get parent UpnpDevice, if any."""
+        return self._parent_device
+
+    @parent_device.setter
+    def parent_device(self, parent_device: "UpnpDevice") -> None:
+        """Set parent UpnpDevice."""
+        if self._parent_device is not None:
+            raise UpnpError("UpnpDevice already bound to UpnpDevice")
+
+        self._parent_device = parent_device
+
+    @property
+    def root_device(self) -> "UpnpDevice":
+        """Get the root device, or self if self is the root device."""
+        if self._parent_device is None:
+            return self
+
+        return self._parent_device.root_device
+
+    def find_device(self, device_type: str) -> Optional["UpnpDevice"]:
+        """Find a (embedded) device with the given device_type."""
+        if self.device_type == device_type:
+            return self
+
+        for embedded_device in self.embedded_devices.values():
+            device = embedded_device.find_device(device_type)
+            if device:
+                return device
+
+        return None
+
+    def find_service(self, service_type: str) -> Optional["UpnpService"]:
+        """Find a service with the give service_type."""
+        if service_type in self.services:
+            return self.services[service_type]
+
+        for embedded_device in self.embedded_devices.values():
+            service = embedded_device.find_service(service_type)
+            if service:
+                return service
+
+        return None
+
+    @property
+    def all_devices(self) -> List["UpnpDevice"]:
+        """Get all devices, self and embedded."""
+        devices = [self]
+
+        for embedded_device in self.embedded_devices.values():
+            devices += embedded_device.all_devices
+
+        return devices
+
+    @property
+    def all_services(self) -> List["UpnpService"]:
+        """Get all services, from self and embedded devices."""
+        services: List["UpnpService"] = []
+
+        for device in self.all_devices:
+            services += device.services.values()
+
+        return services
+
+    def reinit(self, new_device: "UpnpDevice") -> None:
         """Reinitialize self from another device."""
-        self.device_info = device.device_info
+        if self.device_type != new_device.device_type:
+            raise UpnpError(
+                f"Mismatch in device_type: {self.device_type} vs {new_device.device_type}"
+            )
+
+        self.device_info = new_device.device_info
+
+        # reinit embedded devices
+        for device_type, embedded_device in self.embedded_devices.items():
+            new_embedded_device = new_device.embedded_devices[device_type]
+            embedded_device.reinit(new_embedded_device)
 
     @property
     def name(self) -> str:
@@ -180,7 +267,7 @@ class UpnpDevice:
 
     def __str__(self) -> str:
         """To string."""
-        return "<UpnpDevice({0})>".format(self.udn)
+        return f"<UpnpDevice({self.udn})>"
 
 
 class UpnpService:
@@ -333,14 +420,14 @@ class UpnpService:
         udn = "unbound"
         if self._device:
             udn = self._device.udn
-        return "<UpnpService({}, {})>".format(self.service_id, udn)
+        return f"<UpnpService({self.service_id}, {udn})>"
 
     def __repr__(self) -> str:
         """To repr."""
         udn = "unbound"
         if self._device:
             udn = self._device.udn
-        return "<UpnpService({}, {})>".format(self.service_id, udn)
+        return f"<UpnpService({self.service_id}, {udn})>"
 
 
 class UpnpAction:
@@ -413,7 +500,7 @@ class UpnpAction:
 
         def __repr__(self) -> str:
             """To repr."""
-            return "<UpnpAction.Argument({}, {})>".format(self.name, self.direction)
+            return f"<UpnpAction.Argument({self.name}, {self.direction})>"
 
     def __init__(
         self,
@@ -457,13 +544,11 @@ class UpnpAction:
 
     def __str__(self) -> str:
         """To string."""
-        return "<UpnpAction({0})>".format(self.name)
+        return f"<UpnpAction({self.name})>"
 
     def __repr__(self) -> str:
         """To repr."""
-        return "<UpnpAction({0})({1}) -> {2}>".format(
-            self.name, self.in_arguments(), self.out_arguments()
-        )
+        return f"<UpnpAction({self.name})({self.in_arguments()}) -> {self.out_arguments()}>"
 
     def validate_arguments(self, **kwargs: Any) -> None:
         """
@@ -513,8 +598,7 @@ class UpnpAction:
 
         if status_code != 200:
             raise UpnpError(
-                "Error during async_call(), status: %s, body: %s"
-                % (status_code, response_body)
+                f"Error during async_call(), status: {status_code}, body: {response_body}"
             )
 
         # parse body
@@ -532,21 +616,21 @@ class UpnpAction:
         service_type = self.service.service_type
         soap_args = self._format_request_args(**kwargs)
         body = (
-            '<?xml version="1.0"?>'
-            '<s:Envelope s:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/"'
-            ' xmlns:s="http://schemas.xmlsoap.org/soap/envelope/">'
-            "<s:Body>"
-            '<u:{1} xmlns:u="{0}">'
-            "{2}"
-            "</u:{1}>"
-            "</s:Body>"
-            "</s:Envelope>".format(service_type, self.name, soap_args)
+            f'<?xml version="1.0"?>'
+            f'<s:Envelope s:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/"'
+            f' xmlns:s="http://schemas.xmlsoap.org/soap/envelope/">'
+            f"<s:Body>"
+            f'<u:{self.name} xmlns:u="{service_type}">'
+            f"{soap_args}"
+            f"</u:{self.name}>"
+            f"</s:Body>"
+            f"</s:Envelope>"
         )
 
         # construct SOAP header
-        soap_action = "{0}#{1}".format(service_type, self.name)
+        soap_action = f"{service_type}#{self.name}"
         headers = {
-            "SOAPAction": '"{0}"'.format(soap_action),
+            "SOAPAction": f'"{soap_action}"',
             "Host": urllib.parse.urlparse(control_url).netloc,
             "Content-Type": 'text/xml; charset="utf-8"',
             "Content-Length": str(len(body)),
@@ -557,7 +641,7 @@ class UpnpAction:
     def _format_request_args(self, **kwargs: Any) -> str:
         self.validate_arguments(**kwargs)
         arg_strs = [
-            "<{0}>{1}</{0}>".format(arg.name, escape(arg.coerce_upnp(kwargs[arg.name])))
+            f"<{arg.name}>{escape(arg.coerce_upnp(kwargs[arg.name]))}</{arg.name}>"
             for arg in self.in_arguments()
         ]
         return "\n".join(arg_strs)
@@ -578,8 +662,9 @@ class UpnpAction:
             error_code = xml.findtext(".//control:errorCode", None, NS)
             error_description = xml.findtext(".//control:errorDescription", None, NS)
             raise UpnpError(
-                "Error during call_action, error_code: %s, error_description: %s"
-                % (error_code, error_description)
+                f"Error during call_action, "
+                f"error_code: {error_code}, "
+                f"error_description: {error_description}"
             )
 
         try:
@@ -593,18 +678,17 @@ class UpnpAction:
     ) -> Mapping[str, Any]:
         """Parse response arguments."""
         args = {}
-        query = ".//{{{0}}}{1}Response".format(service_type, self.name)
+        query = f".//{{{service_type}}}{self.name}Response"
         response = xml.find(query, NS)
 
         # If no response was found, do a search ignoring namespaces when in non-strict mode.
         if response is None and self._non_strict:
-            query = ".//{{*}}{0}Response".format(self.name)
+            query = f".//{{*}}{self.name}Response"
             response = xml.find(query, NS)
 
         if response is None:
-            raise UpnpError(
-                "Invalid response: %s" % (ET.tostring(xml, encoding="unicode"),)
-            )
+            xml_str = ET.tostring(xml, encoding="unicode")
+            raise UpnpError(f"Invalid response: {xml_str}")
 
         for arg_xml in response.findall("./"):
             name = arg_xml.tag
@@ -613,9 +697,9 @@ class UpnpAction:
                 if self._non_strict:
                     continue
 
+                xml_str = ET.tostring(xml, encoding="unicode")
                 raise UpnpError(
-                    "Invalid response, unknown argument: %s, %s"
-                    % (name, ET.tostring(xml, encoding="unicode"))
+                    f"Invalid response, unknown argument: {name}, {xml_str}"
                 )
 
             arg.raw_upnp_value = arg_xml.text
@@ -803,4 +887,4 @@ class UpnpStateVariable(Generic[T]):
 
     def __str__(self) -> str:
         """To string."""
-        return "<UpnpStateVariable({0}, {1})>".format(self.name, self.data_type)
+        return f"<UpnpStateVariable({self.name}, {self.data_type})>"
